@@ -18,30 +18,38 @@ from ranger.ext.accumulator import Accumulator
 from ranger.ext.lazy_property import lazy_property
 from ranger.ext.human_readable import human_readable
 from ranger.container.settings import LocalSettings
+from ranger.ext.vcs import Vcs
+
 
 def sort_by_basename(path):
     """returns path.relative_path (for sorting)"""
     return path.relative_path
 
+
 def sort_by_basename_icase(path):
     """returns case-insensitive path.relative_path (for sorting)"""
     return path.relative_path_lower
 
+
 def sort_by_directory(path):
     """returns 0 if path is a directory, otherwise 1 (for sorting)"""
     return 1 - path.is_directory
 
+
 def sort_naturally(path):
     return path.basename_natural
 
+
 def sort_naturally_icase(path):
     return path.basename_natural_lower
 
+
 def sort_unicode_wrapper_string(old_sort_func):
     def sort_unicode(path):
         return locale.strxfrm(old_sort_func(path))
     return sort_unicode
 
+
 def sort_unicode_wrapper_list(old_sort_func):
     def sort_unicode(path):
         return [locale.strxfrm(str(c)) for c in old_sort_func(path)]
@@ -62,6 +70,7 @@ def accept_file(file, filters):
             return False
     return True
 
+
 def walklevel(some_dir, level):
     some_dir = some_dir.rstrip(os.path.sep)
     followlinks = True if level > 0 else False
@@ -73,6 +82,7 @@ def walklevel(some_dir, level):
         if level != -1 and num_sep + level <= num_sep_this:
             del dirs[:]
 
+
 def mtimelevel(path, level):
     mtime = os.stat(path).st_mtime
     for dirpath, dirnames, filenames in walklevel(path, level):
@@ -81,6 +91,7 @@ def mtimelevel(path, level):
         mtime = max(mtime, max([-1] + [os.stat(d).st_mtime for d in dirlist]))
     return mtime
 
+
 class Directory(FileSystemObject, Accumulator, Loadable):
     is_directory = True
     enterable = False
@@ -109,6 +120,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
     content_outdated = False
     content_loaded = False
 
+    vcs = None
     has_vcschild = False
 
     _cumulative_size_calculated = False
@@ -145,6 +157,9 @@ class Directory(FileSystemObject, Accumulator, Loadable):
 
         self.settings = LocalSettings(path, self.settings)
 
+        if self.settings.vcs_aware:
+            self.vcs = Vcs(self)
+
         self.use()
 
     def request_resort(self):
@@ -159,7 +174,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
     def mark_item(self, item, val):
         item._mark(val)
         if val:
-            if item in self.files and not item in self.marked_items:
+            if item in self.files and item not in self.marked_items:
                 self.marked_items.append(item)
         else:
             while True:
@@ -209,7 +224,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
 
     def refilter(self):
         if self.files_all is None:
-            return # propably not loaded yet
+            return  # propably not loaded yet
 
         self.last_update_time = time()
 
@@ -229,7 +244,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
             filters.append(lambda file: temporary_filter_search(file.basename))
 
         self.files = [f for f in self.files_all if accept_file(f, filters)]
-        
+
         # A fix for corner cases when the user invokes show_hidden on a
         # directory that contains only hidden directories and hidden files.
         if self.files and not self.pointed_obj:
@@ -301,10 +316,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
                 files = []
                 disk_usage = 0
 
-                if self.settings.vcs_aware:
-                    self.has_vcschild = False
-                    self.load_vcs(None)
-
+                has_vcschild = False
                 for name in filenames:
                     try:
                         file_lstat = os_lstat(name)
@@ -314,38 +326,43 @@ class Directory(FileSystemObject, Accumulator, Loadable):
                             file_stat = file_lstat
                         stats = (file_stat, file_lstat)
                         is_a_dir = file_stat.st_mode & 0o170000 == 0o040000
-                    except:
+                    except Exception:
                         stats = None
                         is_a_dir = False
                     if is_a_dir:
-                        if self.flat:
+                        try:
+                            item = self.fm.get_directory(name)
+                            item.load_if_outdated()
+                        except Exception:
                             item = Directory(name, preload=stats, path_is_abs=True,
-                                    basename_is_rel_to=basename_is_rel_to)
+                                             basename_is_rel_to=basename_is_rel_to)
                             item.load()
                         else:
-                            try:
-                                item = self.fm.get_directory(name)
-                                item.load_if_outdated()
-                            except:
-                                item = Directory(name, preload=stats, path_is_abs=True)
-                                item.load()
+                            if self.flat:
+                                item.relative_path = os.path.relpath(item.path, self.path)
+                            else:
+                                item.relative_path = item.basename
+                            item.relative_path_lower = item.relative_path.lower()
+                        if item.vcs and item.vcs.track:
+                            if item.vcs.is_root_pointer:
+                                has_vcschild = True
+                            else:
+                                item.vcsstatus = item.vcs.rootvcs.status_subpath(
+                                    os.path.join(self.realpath, item.basename), is_directory=True)
                     else:
                         item = File(name, preload=stats, path_is_abs=True,
                                     basename_is_rel_to=basename_is_rel_to)
                         item.load()
                         disk_usage += item.size
-
-                    # Load vcs data
-                    if self.settings.vcs_aware:
-                        item.load_vcs(self)
-                        if item.vcs_enabled:
-                            self.has_vcschild = True
+                        if self.vcs and self.vcs.track:
+                            item.vcsstatus = self.vcs.rootvcs.status_subpath(
+                                os.path.join(self.realpath, item.basename))
 
                     files.append(item)
                     self.percent = 100 * len(files) // len(filenames)
                     yield
+                self.has_vcschild = has_vcschild
                 self.disk_usage = disk_usage
-                self.vcs_outdated = False
 
                 self.filenames = filenames
                 self.files_all = files
@@ -378,6 +395,8 @@ class Directory(FileSystemObject, Accumulator, Loadable):
         finally:
             self.loading = False
             self.fm.signal_emit("finished_loading_dir", directory=self)
+            if self.vcs:
+                self.fm.ui.vcsthread.process(self)
 
     def unload(self):
         self.loading = False
@@ -416,7 +435,6 @@ class Directory(FileSystemObject, Accumulator, Loadable):
                     pass
                 self.load_generator = None
 
-
     def sort(self):
         """Sort the contained files"""
         if self.files_all is None:
@@ -424,7 +442,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
 
         try:
             sort_func = self.sort_dict[self.settings.sort]
-        except:
+        except Exception:
             sort_func = sort_by_basename
 
         if self.settings.sort_case_insensitive and \
@@ -442,13 +460,13 @@ class Directory(FileSystemObject, Accumulator, Loadable):
             elif sort_func in (sort_by_basename, sort_by_basename_icase):
                 sort_func = sort_unicode_wrapper_string(sort_func)
 
-        self.files_all.sort(key = sort_func)
+        self.files_all.sort(key=sort_func)
 
         if self.settings.sort_reverse:
             self.files_all.reverse()
 
         if self.settings.sort_directories_first:
-            self.files_all.sort(key = sort_by_directory)
+            self.files_all.sort(key=sort_by_directory)
 
         self.refilter()
 
@@ -466,7 +484,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
                     else:
                         stat = os_stat(dirpath + "/" + file)
                     cum += stat.st_size
-                except:
+                except Exception:
                     pass
         return cum
 
@@ -520,7 +538,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
     def move_to_obj(self, arg):
         try:
             arg = arg.path
-        except:
+        except Exception:
             pass
         self.load_content_once(schedule=False)
         if self.empty():
@@ -535,10 +553,10 @@ class Directory(FileSystemObject, Accumulator, Loadable):
         length = len(self)
 
         if forward:
-            generator = ((self.pointer + (x + offset)) % length \
+            generator = ((self.pointer + (x + offset)) % length
                     for x in range(length - 1))
         else:
-            generator = ((self.pointer - (x + offset)) % length \
+            generator = ((self.pointer - (x + offset)) % length
                     for x in range(length - 1))
 
         for i in generator:
@@ -569,7 +587,7 @@ class Directory(FileSystemObject, Accumulator, Loadable):
         try:
             if self == self.fm.thisdir:
                 self.fm.thisfile = self.pointed_obj
-        except:
+        except Exception:
             pass
 
     def load_content_once(self, *a, **k):
@@ -582,7 +600,8 @@ class Directory(FileSystemObject, Accumulator, Loadable):
     def load_content_if_outdated(self, *a, **k):
         """Load the contents of the directory if outdated"""
 
-        if self.load_content_once(*a, **k): return True
+        if self.load_content_once(*a, **k):
+            return True
 
         if self.files_all is None or self.content_outdated:
             self.load_content(*a, **k)
